#!/bin/bash
# ------------------------------------------
# Datei  :  bak.sh
# Autor  :  Ferdinand Gruber, fer.gru@aon.at
# ------------------------------------------

usage() {
    echo "-----------------------"
    echo "bak.sh - Datensicherung"
    echo "-----------------------"
    echo "Parameter:"
    echo "   auto, rotation, testRotation, mysqldump, testlauf, shutdown, suspend, USB, SSH, NFS"
    echo "   Es sind auch Wertepaare als Parameter erlaubt."
    echo        "shutdown='true'"
    echo        "exclude='/filePath'"
    echo "-----------------------"
    echo "Optionen:"
    echo "   -v  Es werden möglichst viele Meldungen ausgegeben"
    echo "   --help | --usage: Dieser Text"
    echo "-----------------------"
    echo "Aufruf:"
    echo "$0 [Optionen] [Parameter] [Parameter] [Parameter] ..."
    echo "-----------------------"
    echo "Beispiele:"
    echo "   $0 --help (Dieser Text wird angezeigt)"
    echo "   $0 testlauf: Backup wird simuliert, es wird nichts kopiert"
    echo "   $0 shutdown: Herunterfahren des Computers nach dem Backup"
    echo "   $0 auto: Programm wird mit Standardoptionen gestartet"
    echo "       Die Optionen sind in 'bak.conf' definiert"
    echo "---------------------------------------------------------"
    echo
    read -s -n1 -p "Programm starten (Parameter: auto) mit beliebiger Taste, [n] für Abbruch. " keypress
    echo

    if [ $keypress ] ; then
        if [ $keypress = "n" ] ; then exit ; fi
    else
        $0 auto
        exit
    fi
}

rotation () {
    echo "--------------------------------------------------------------------------"
    if [ $action = "testlauf" ] ; then
        echo "Keine Rotation der Sicherungssätze beim Testlauf"
        echo ssh -p $PORT $ZIEL_Host "nohup /usr/local/bin/rotation.sh $ZIEL_Verzeichnis $CounterRotation > $redirection &"
        echo "--------------------------------------------------------------------------"
    else
        if [ $CONNECTION = "SSH" ] ; then
            echo "Rotation gestartet auf dem Host $ZIEL_Host: `date "+%d.%m.%Y um %H:%M"`"
            if [ $ZIEL_Host != "localhost" ] ; then
                redirection="/var/log/bak_rotation_$SUBDIR.log"
                ssh -p $PORT ${USER}@$ZIEL_Host "nohup /usr/local/bin/rotation.sh $ZIEL_Verzeichnis $CounterRotation > $redirection &"
            else
                $workDir/rotation.sh $ZIEL_Verzeichnis $CounterRotation
            fi
        else
            $workDir/rotation.sh $ZIEL_Verzeichnis $CounterRotation
        fi
    fi
}

verify_CONNECTION () {
    cp $ZIEL_Verzeichnis/testfile /dev/null &> $redirection
    if [ $? -gt 0 ] ; then
      scp $ZIEL_Verzeichnis/testfile /dev/null &> $redirection
      if [ $? -gt 0  ] ; then
        if [ -z $CONNECTION ] ; then
            msg="Das Ziel '$ZIEL_Pfad' ist nicht erreichbar";
        else
            msg="Das Ziel '$ZIEL_Pfad' ist über $CONNECTION nicht erreichbar";
        fi
        echo "--------------------------------------------------------------------------"
        echo "$msg bzw. die Datei $ZIEL_Verzeichnis/testfile existiert nicht."
        echo "Backup abgebrochen."
        echo "--------------------------------------------------------------------------"
        exit 1
      fi
    fi
}

do_backup () {
    StartTime=$(date "+%d.%m.%Y um %H:%M")

    echo "--------------------------------------------------------------------------"
    echo $0
    echo "Parameter:" $*
    echo "--------------------------------------------------------------------------"
    echo "Sicherung von $QUELL_Pfad auf: $ZIEL_Pfad"
    echo "Connection: "$CONNECTION
    echo "Rsync Optionen: $defaultOptions $EXTRAOPT"
    echo "Beginn der Sicherung: $StartTime"
    echo

    # Liste von Dateipattern, die nicht gesichert werden sollen
    FILE_excludes=$workDir/excludes.txt
    FILE_includes=$workDir/includes.txt

        # Abwärtscompatibilitär 27.11.2021
        if [ ! -f $FILE_excludes ] ; then
            FILE_exludes=$workDir/localhost_excludes.txt
        fi
        if [ ! -f $FILE_includes ] ; then
            FILE_includes=$workDir/localhost_includes.txt
        fi

    EXCLUDES="--exclude-from="$FILE_excludes
    INCLUDES="--include-from="$FILE_includes

    case $CONNECTION in
        "NFS")
            # NFS Mount versuchen
            showmount $HOST &> $redirection && sudo mount -t nfs -o nfsvers=3,noauto $HOST:$MOUNTPOINT $MOUNTPOINT 2> $redirection

            if [ "$(/usr/sbin/nfsstat -m | grep $HOST:$MOUNTPOINT)" ] ; then # wenn NFS Share gemountet ist
            HOST=localhost
            echo "--------------------------------------------------------------------------"
            echo Sicherung auf NFS Share $HOST:$MOUNTPOINT
            else
                echo Kann $ZIEL_Verzeichnis nicht über NFS mounten, daher Abbruch.
                exit 1
            fi
            verify_CONNECTION
            mkdir $MOUNTPOINT/$ZIEL_Verzeichnis/backup.0 2> /dev/null
            touch -c $ZIEL_Verzeichnis/backup.0
            if ( $? = 0 ) ; then
                rsync $defaultOptions $EXTRAOPT --protect-args --numeric-ids --delete $INCLUDES $EXCLUDES \
                    $QUELL_Pfad $ZIEL_Pfad/ | tee -a $LOGFILE 2> $redirection
                touch $ZIEL_Verzeichnis/backup.0
            else
                exit 1
            fi
            ;;
        "SSH")
            if [ $QUELL_Host = "localhost" ] ; then ZIEL_Pfad=${USER}@$ZIEL_Host:$ZIEL_Verzeichnis/backup.0 ; fi
            if [ $ZIEL_Host = "localhost" ] ; then QUELL_Pfad=${USER}@$QUELL_Host:$QUELL_Pfad ; fi
            rsync $defaultOptions $EXTRAOPT --protect-args --numeric-ids -e "ssh -p $PORT" --delete $INCLUDES $EXCLUDES \
                $QUELL_Pfad $ZIEL_Pfad/ | tee -a $LOGFILE 2> $redirection

            ssh -p $PORT ${USER}@$ZIEL_Host 'mkdir $ZIEL_Verzeichnis 2> /dev/null'
            ssh -p $PORT ${USER}@$ZIEL_Host 'mkdir $ZIEL_Verzeichnis/backup.0 2> /dev/null'
            ssh -p $PORT ${USER}@$ZIEL_Host 'touch -c $ZIEL_Verzeichnis/backup.0'
            ;;
        "LOCAL" | "USB")
            mkdir $MOUNTPOINT 2> /dev/null
            mkdir $MOUNTPOINT/$ZIEL_Verzeichnis 2> /dev/null
            touch -c $MOUNTPOINT/$ZIEL_Verzeichnis/backup.0
            mount $DEVICE $MOUNTPOINT 2> $redirection
            rsync $defaultOptions $EXTRAOPT --protect-args --numeric-ids --delete $INCLUDES $EXCLUDES \
                $QUELL_Pfad $ZIEL_Pfad/ | tee -a $LOGFILE 2> $redirection
            ;;
    esac
}

############################
# Hauptprogramm
############################
if [ $(id -u) -gt 0 ] ; then # detect user root
    echo "-------------------------------------------"
    echo "Programm abgebrochen"
    echo "Backup ist nur als Benutzer 'root' möglich."
    echo "Der angemeldete Benutzer ist '"$(whoami)"'"
    echo "-------------------------------------------"
    exit
fi

# Variablen Standardwerte
CRON="false"
EXTRAOPT=
LOG="false"
QUELL_Host="localhost"
ROTATION="false"
USER=root # SSH user
ZIEL_Host="localhost"

action="none"
auto="false"
diskLabel="LinuxBackup"
exclude=
mysqlPassword="xxxxxxxx"
mysqldump="false"
mysqlfiles="false"
redirection="/dev/null"
showInfo="false"
testRotation="false"

workDir="undefined"
if [ -f $(dirname $0)/bak.conf ] ; then
    workDir=$(dirname $0)
else
    if [ -L $0 ] ; then
        file=$(readlink $0)
        dir=$(dirname $file)
        if [ $(echo $dir | cut -c1) = "/" ] ; then
            # absolouter Pfad
            workDir=$dir
        else
            workDir=$(dirname $0)"/"$dir
        fi
    fi
fi

if  [ $workDir = "undefined" ] ; then
    echo "Keine Konfigurationsdatei in" $workDir "vorhanden."
    exit 1
fi

# Eingabeparameter auswerten
configParams=
if [ $# -gt 0 ] ; then
    for param in $@ ; do
        case $param in
            showinfo) showInfo="true";;
            cron) showInfo="false";CRON="true";;
            mysqldump) mysqldump="true";;
            mysqlfiles) mysqlfiles="true";;

            shutdown) action=$param;;
            suspend) action=$param;;

            SSH) CONNECTION=$param;;
            NFS) CONNECTION=$param;;
            # RSYNCD) CONNECTION=$param;;
            USB) CONNECTION=$param;;

            rotation) ROTATION="true";;
            testlauf) EXTRAOPT+="--dry-run "; showInfo="false"; action="testlauf";;
            testRotation) testRotation="true"; mysqldump="false";mysqlfiles="false";;
            verbose | -v)
                redirection="/dev/stdout"
                LOG="true"
                ;;
            *)
                # Wertepaare in Variablen einlesen - experimentell
                if [ $(echo $param | grep -c "=") = 1 ] ; then
                    key=$(echo $param | cut -d "=" -f "1")
                    val=$(echo $param | cut -d "=" -f "2")
                    declare $key=$val
                fi
                ;;
        esac
    done
else
    showInfo="true"
fi

if [ $showInfo = "true" ] ; then
    usage
fi
if [ $exclude ] ; then
    EXTRAOPT+=" --exclude="$exclude
fi

# ---------------------------------------
# Konfiguration laden
# ---------------------------------------
# Eingabeparameter an bak.conf übergeben
source $workDir/bak.conf
# ---------------------------------------

if [ $ZIEL_Host = $QUELL_Host ] ; then CONNECTION="LOCAL" ; fi

if [ -z $CONNECTION ] ; then
    # autodetect USB DEVICE
    UUID=$(/sbin/blkid | grep $diskLabel | cut -d " " -f 3 | cut -d "\"" -f 2)
    DEVICE="/dev/disk/by-uuid/"$UUID
    if [ -b $DEVICE ] ; then
        CONNECTION="USB"
    else
        CONNECTION="SSH"
    fi
fi

if [ -z $SUBDIR ] ; then
    SUBDIR=$(hostname | cut -f 1 -d ".")
fi

ZIEL_Verzeichnis=$MOUNTPOINT/$SUBDIR
ZIEL_Pfad=$ZIEL_Verzeichnis/backup.0

if [ $LOG = "true" ] ; then
    LOGFILE="/var/log/bak_"$QUELL_Host"-"$ZIEL_Host":"$SUBDIR".log"
else
    LOGFILE="/dev/null"
fi

# ******************
# Backup durchführen
# ******************
if [ $mysqldump = "true" ] ; then
    echo "Mysql Datenbank Backup"
    if [ $mysqlfiles = "true"  ]  ; then
        $workDir/mysql_bak.sh -p $mysqlPassword "dump" "files"
    else
        $workDir/mysql_bak.sh -p $mysqlPassword "dump"
    fi
fi

if [ $testRotation = "true" ] ; then
    rotation
    exit
else
    case $CONNECTION in
        "SSH" | "NFS")
            defaultOptions="-HXavz"
            ;;
        *)
            defaultOptions="-HXav"
            ;;
    esac
    do_backup $*
fi

# Rueckgabewert pruefen.
# 24 ist harmlos; tritt auf, wenn waehrend der Laufzeit
# von Rsync noch (/tmp?)-Dateien verändert oder geloescht wurden.
# Alles andere ist fatal -- siehe man (1) rsync
errorCode=$?
if [ $errorCode = 24 ] ; then errorCode=0 ; fi

if [ $errorCode -gt 0 ] ; then
    echo "rsync vorzeitig beendet mit Errorcode: $errorCode" | tee $LOGFILE
    echo $(date "+%d.%m.%Y um %H:%M") | tee -a $LOGFILE
else
    sync
    echo "Fertig mit Backup von $QUELL_Pfad nach $ZIEL_Pfad" | tee $LOGFILE
    echo $(date "+%d.%m.%Y um %H:%M") | tee -a $LOGFILE
    if [ $ROTATION = "true" ] ; then
        rotation
    else
        echo "Keine Rotation"  | tee | tee -a $LOGFILE
        echo "Die Variable ROTATION hat den Wert: $ROTATION"  | tee -a $LOGFILE
    fi
fi
echo "--------------------------------------------------------------------------"

if [ $errorCode -gt 0 ] ; then
    mail -s "Datensicherung - $QUELL_Host" root < $(tail $LOGFILE);
fi

case $action in
    shutdown) echo "Der Rechner wird in 3 Minuten heruntergefahren."
        /sbin/shutdown -h 3
        ;;
    suspend) echo "Der Rechner wird in Tiefschlaf versetzt."
        /usr/bin/powersave -U
        ;;
esac

exit $errorCode

