#!/bin/bash
# ------------------------------------------
# Datei   :  bak_cron.sh
# Zweck   :  Steuerung der Datensicherung über cronjob
# Autor   :  Ferdinand Gruber, fer.gru@aon.at
# ------------------------------------------

MAXCOUNTER=3
STATUSFILE="/tmp/bak.sh-status"
LOCKFILE="/tmp/backup.lock"

verbose="none"
force="false"
redirection="/dev/null"
LOG="/dev/null"

while getopts ":vf" options ; do
    case $options in
    v )
        verbose="verbose"
        redirection="/dev/stdout"
        LOG="/var/log/bak_cron.log"
        ;;
    f ) # Sofort sichern, keine Warteschleife
        MAXCOUNTER=0
        force="true"
        ;;
    \? )
      echo "Ungültige Option: -$OPTARG" >&2
      ;;
    esac
done

# LOCKFILE löschen, wenn älter als 6 Stunden
if [ -f $LOCKFILE ] ; then
    date_lockfile=$(stat -c  %Y $LOCKFILE)
    date_jetzt=$(date +%s)
    date_diff=$(expr $date_jetzt - $date_lockfile)
    if [ $date_diff -gt 21600 ] ; then
        rm $LOCKFILE
    fi
fi

touch $STATUSFILE;
chmod 666 $STATUSFILE
read COUNTER < $STATUSFILE
if [ -z $COUNTER ] ;
    then COUNTER=0
fi

desktopList="kwin openbox"
readyToStart="true"
for desktop in $desktopList ; do
	if [ `ps -e | grep $desktop | wc -l` -gt 0 ] ; then readyToStart="false" ; fi
done
AnzahlUser=$(who | grep -c "tty")
if [ $AnzahlUser -gt 0 ] ; then readyToStart="false" ; fi

if [ $readyToStart = "true" -o $force = "true" ] ; then
    # Sicherung vorbereiten, da kein Benutzer mehr angemeldet ist.
    if [ $COUNTER -ge $MAXCOUNTER ] ; then
        if [ -f $LOCKFILE ] ; then
            echo "$(date "+%d.%m.%Y um %H:%M"): Abbruch, da $LOCKFILE existiert" | tee -a $LOG &> $redirection
            exit 1
        fi

        # pkill Xorg
        touch /etc/nologin
        sleep 5s

        # Sicherung durchführen
        touch $LOCKFILE
        chmod 666 $LOCKFILE

        /usr/local/bin/bak.sh "$verbose $@ cron"

        if [ $force != "true" ] ; then
            /sbin/shutdown -h 3
        fi
    else
        if [ -f $LOCKFILE ] ; then
            # rm $LOCKFILE &> $redirection
              rm $LOCKFILE | tee -a $LOG &> $redirection
        fi
        expr $COUNTER + 1 > $STATUSFILE
        echo "Statusfile gespeichert:" `date "+%d.%m.%Y um %H:%M"` " - COUNTER: " $COUNTER | tee -a $LOG &> $redirection
    fi
else
    if [ -f $LOCKFILE ] ; then
        # rm $LOCKFILE &> $redirection
        rm $LOCKFILE | tee -a $LOG &> $redirection
    fi
    if [ -f $STATUSFILE ] ; then
        # rm $STATUSFILE &> $redirection
        rm $STATUSFILE | tee -a $LOG &> $redirection
    fi
    echo "Keine Sicherung durchgeführt (\$readytToStart = 'false')" | tee -a $LOG &> $redirection
    echo "Anzahl User: $AnzahlUser" | tee -a $LOG &> $redirection
fi

echo "bak_cron.sh Leerlauf: $(date "+%d.%m.%Y um %H:%M")"  | tee -a $LOG &> $redirection
echo Anzahl User: $AnzahlUser  | tee -a $LOG &> $redirection

