#!/bin/bash
# ------------------------------------------
# Datei   :  mysql_bak.sh
# Autor   :  Ferdinand Gruber
# Datum   :  20.06.2013
# Zweck   :  Mysql Datenbanksicherung
# Kontext :  Datensicherung
# Info:      http://wiki.ubuntuusers.de/MySQL
# Eingabeparameter:
#   "-p <Mysql Passwort>
#   "dump" - Sicherung als Mysqldump
#   "files" - Sicherung des Verzeichnisses /var/lib/mysql aus /var/lib/mysq_bak
# -----------------------------------------------------------------------------

dump="true"
files="true"

if [ -d "/srv/www/htdocs" ] ; then
    documenRoot="/srv/www/htdocs"
else
    documenRoot="/var/www/html"
fi
backup_dir=$documenRoot"/mysqldump"

user="root"
password="xxxxxxxxxxx"

while getopts "p:" options ; do
    case $options in
        p) password="$OPTARG" ;;
    esac
    shift $((OPTIND -1)) # Optionen aus Parameterliste entfernen
done

if [ $# -gt 0 ] ; then
    dump="false"
    files="false"
    case $1 in
        dump) dump="true" ;;
        files) files="true" ;;
    esac
    shift
fi

if [ $dump = "true" ] ; then
    # Sicherung aller Datenbanken in einer Datei
    mkdir $backup_dir &> /dev/null
    mysqldump --opt --user=$user --password=$password --all-databases --result-file=$backup_dir/mysqldump.sql

    # Sicherung jeder DB in einer eigenen Datei
    mkdir $backup_dir/databases &> /dev/null
    for db in `mysqlshow -u $user -p$password | sed -n -e "s/^| \([^ ]*\)[ ]*|$/\1/p"` ; do
        # echo $db
        if [ $db = performance_schema -o $db = information_schema ] ; then
            continue
        fi
        mysqldump --opt --single-transaction --user=$user --password=$password $db > $backup_dir/databases/$db.sql
    done
    chown root.root $backup_dir/* -R
    chmod 755 $backup_dir/* -R
    if [ $? -eq 0 ] ; then
        echo "Mysqldump gespeichert in "$backup_dir
    else
        echo "Fehler beime Mysqldump"
    fi
fi

if [ $files = "true" ] ; then
    # Sicherung der Mysql Datenbank auf Dateisystembasis
    echo "/var/lib/mysql wird kopiert nach /var/lib/mysql_bak "
    systemctl stop apache2.service
    systemctl stop mysql.service
    rsync -av --delete /var/lib/mysql/* /var/lib/mysql_bak > /dev/null
    systemctl start mysql.service
    systemctl start apache2.service
fi

