#!/bin/bash
# -------------------------------------------------------
# Datei   :  rotation.sh
# Autor   :  Ferdinand Gruber
# Datum   :  09.02.2014
# Zweck   :  Backup Rotation
# Kontext :  Datensicherung
# Quelle:    https://www.heinlein-support.de/howto/backups-und-snapshots-von-linux-servern-mit-rsync-und-ssh
#            modifiziert von Ferdinand Gruber
# -------------------------------------------------------
# Parameter: Zielverzeichnis, Anzahl der Sicherungsstufen
# -------------------------------------------------------


if [ $# -lt 2  ] ; then 
    echo "Abbruch, da 2 Parameter fehlen."
    exit 1 
else 
    ZIEL_Verzeichnis=$1
    MAX=$2
fi

logFile="/var/log/bak_rotation_${ZIEL_Verzeichnis}.log"

# if [ -f $logFile ] ; then
#     rm $logFile > /dev/null
# fi

echo
echo ZIEL_Verzeichnis: $ZIEL_Verzeichnis
echo $MAX Backupverzeichnisse rotieren
echo "--------------------------------------------------------------------------"
echo "Beginn der Rotation: `date "+%d.%m.%Y um %H:%M"`"
echo

# backup.1 bis backup.9 umbenennen (Abwärtskompatibilität)
cd $ZIEL_Verzeichnis
for d in $(seq 1 9) ; do
    if [ -d backup.$d ] ; then
        mv backup.$d backup.0$d
    fi
done

# Verzeichnis anfassen, um Backup-Datum zu speichern
touch $ZIEL_Verzeichnis/backup.0

# Das hoechste Snapshot loeschen
if [ -d $ZIEL_Verzeichnis/backup.$MAX ] ; then
    rm -rf $ZIEL_Verzeichnis/backup.$MAX
fi

# Alle anderen Snapshots eine Nummer nach oben verschieben
OLD=$(expr $MAX - 1)
while [ $OLD -ge 1 ] ; do
    nameOLD=$(printf '%02d' $OLD)
    NEW=$(expr $OLD + 1)
    nameNEW=$(printf '%02d' $NEW)
    if [ -d $ZIEL_Verzeichnis/backup.$nameOLD ] ; then
        touch $ZIEL_Verzeichnis/.timestamp -r $ZIEL_Verzeichnis/backup.$nameOLD
        mv $ZIEL_Verzeichnis/backup.$nameOLD $ZIEL_Verzeichnis/backup.$nameNEW
        touch $ZIEL_Verzeichnis/backup.$nameNEW -r $ZIEL_Verzeichnis/.timestamp
  fi
  OLD=$(expr $OLD - 1)
done

# Snapshot von Level 0 per hardlink-Kopie nach Level 1 kopieren
if [ -d $ZIEL_Verzeichnis/backup.0 ] ; then
  mkdir $ZIEL_Verzeichnis/backup.01 &> /dev/null
  cp -al $ZIEL_Verzeichnis/backup.0/* $ZIEL_Verzeichnis/backup.01
fi

echo "Fertig mit Rotation: `date "+%d.%m.%Y um %H:%M"`"

if [ -f $logFile ] ; then
    mail -s "Datensicherung - Rotation" root < $logFile
fi
